﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using Microsoft.SharePoint;
using System.Data.SqlClient;

namespace JustAddCode.PackageAnything.SQLReceiver
{
    public class SQLReceiver : SPFeatureReceiver
    {
        #region Properties

        private string ConnectionString
        {
            get { return CurrentProperties.Definition.Properties["ConnectionString"].Value;}
        }
        private SPFeatureReceiverProperties CurrentProperties = null;
        private string ActivationScriptString
        {
            get { return CurrentProperties.Definition.Properties["ActivateScript"].Value; }
        }
        private string[] ActivationScripts
        {
            get { return ActivationScriptString.Split(','); }
        }
        private string DeactivationScriptString
        {
            get { return CurrentProperties.Definition.Properties["DeactivateScript"].Value; }
        }
        private string[] DeactivationScripts
        {
            get { return DeactivationScriptString.Split(','); }
        }

        #endregion
        
        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            try
            {
                InitFeature(properties);
                
                RunScripts(ActivationScripts);
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message + " " + ex.StackTrace);
            }
        }

        private void InitFeature(SPFeatureReceiverProperties properties)
        {
            CurrentProperties = properties;
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            try
            {
                InitFeature(properties);

                RunScripts(DeactivationScripts);
                
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message + " " + ex.StackTrace);
                SPException newEx = new SPException(ex.Message);
                throw newEx;
            }
        }

        #region Methods


        private void RunScripts(string[] fileNames)
        {
            foreach (string file in fileNames)
            {
                RunScript(System.IO.Path.Combine(CurrentProperties.Feature.Definition.RootDirectory, file));
            }
        }


        private void RunScript(string filename)
        {
            SqlCommand command = new SqlCommand();
            try
            {
                TextReader reader;
                string scriptToRun = "";
                

                if (File.Exists(filename))
                {
                    scriptToRun = File.ReadAllText(filename);                    
                    string[] commands = scriptToRun.Split(new string[] { "GO" }, StringSplitOptions.RemoveEmptyEntries);

                    command.Connection = new SqlConnection(ConnectionString);
                    command.Connection.Open();

                    foreach (string cmd in commands)
                    {
                        command.CommandText = cmd;
                        command.ExecuteNonQuery();
                    }
                }
            }
            catch (SqlException ex)
            {
                throw ex;
            }
            catch (Exception ex)
            {
                System.Diagnostics.Debug.WriteLine(ex.Message + " " + ex.StackTrace + " for file " + filename);
            }
            finally
            {
                command.Connection.Close();
            }
        }

        #endregion

        #region Unused
        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        {
            // Not used
        }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        {
            // Not used
        }
        #endregion

    }
}
