﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml.Linq;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;

namespace JustAddCode.PackageAnything.WebConfig
{
    public class WebConfigFeatureReceiver : Microsoft.SharePoint.SPFeatureReceiver
    {
        private const string CONFIG_XML_FILE = "WebConfigModifications.xml";
        private SPFeatureReceiverProperties _featureProperties;
        private bool IsInitialized = false;

        private SPWebApplication _currentWebApp = null;
        public SPWebApplication CurrentWebApp
        {
            get
            {
                if (_currentWebApp == null && IsInitialized)
                    _currentWebApp = _featureProperties.Feature.Parent as SPWebApplication;
                
                return _currentWebApp;
            }
        }

        private void InitializeReciever(SPFeatureReceiverProperties properties)
        {
            _featureProperties = properties;
            IsInitialized = true;
        }

        public override void FeatureActivated(SPFeatureReceiverProperties properties)
        {
            System.Diagnostics.Debug.WriteLine("Feature Activated - Begin");
            InitializeReciever(properties);
            string configXmlFile = System.IO.Path.Combine(properties.Feature.Definition.RootDirectory, CONFIG_XML_FILE);

            XDocument doc = XDocument.Load(configXmlFile);

            var owner = doc.Descendants("WebConfigModifications").First().Attribute("Owner");
            var configItemList = from c in doc.Descendants("ConfigModification")
                      select new ConfigModification()
                                 {
                                     Name = c.Attribute("Name"),
                                     Owner = owner,
                                     Path = c.Attribute("Path"),
                                     Sequence = c.Attribute("Sequence"),
                                     ConfigValue = c.Value
                                 };
         
            foreach (var configItem in configItemList)
            {
                SPWebConfigModification configMod = configItem.ToSPWebConfigModification();
                if(!CurrentWebApp.WebConfigModifications.Contains(configMod))
                    CurrentWebApp.WebConfigModifications.Add(configMod);                
            }

            CurrentWebApp.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications(); 
            CurrentWebApp.Update();

            System.Diagnostics.Debug.WriteLine("Feature Activated - End");
        }

        public override void FeatureDeactivating(SPFeatureReceiverProperties properties)
        {
            System.Diagnostics.Debug.WriteLine("Feature Deactivated - Begin");
            InitializeReciever(properties);

            string configXmlFile = System.IO.Path.Combine(properties.Feature.Definition.RootDirectory, CONFIG_XML_FILE);
            XDocument doc = XDocument.Load(configXmlFile);
            var owner = doc.Descendants("WebConfigModifications").First().Attribute("Owner");

            foreach (var configModification in CurrentWebApp.WebConfigModifications)
            {
                if (configModification.Owner == owner.Value)
                    CurrentWebApp.WebConfigModifications.Remove(configModification);
            }

            CurrentWebApp.Farm.Services.GetValue<SPWebService>().ApplyWebConfigModifications();
            CurrentWebApp.Update();

            System.Diagnostics.Debug.WriteLine("Feature Deactivated - End");
        }

        public override void FeatureInstalled(SPFeatureReceiverProperties properties)
        { }

        public override void FeatureUninstalling(SPFeatureReceiverProperties properties)
        { }
    }
}
